ROLL(Regular Omega Language Learning)

This is a library for regular omega language learning.
For more information, please visit our website http://iscasmc.ios.ac.cn/roll/.


Following are command line usage.
==== Help ====

ROLL can be started using java -jar ROLL.jar,  which is equal to the following command.

<code>
java -jar ROLL.jar -h
</code>

==== Running Modes ====

There are three running modes in ROLL, namely test mode, interactive mode and automatic mode.

    *  test mode with -t n k. With this mode, you can test any algorithm integrated in the tool with n randomly generated Büchi automata of k states.
    *  interactive mode with -int. In this mode, you can teach the tool by yourself.
    *  automatic mode with -aut. With this mode, we by default learn a finite automaton with [[http://www.brics.dk/automaton/|DK package]] answering all membership and equivalence queries, and we can also learn a Büchi automaton with the help of [[http://www.languageinclusion.org/doku.php?id=tools|RABIT tool]]. All output messages from RABIT tool can be found in rabitlog.txt file.

The command to run ROLL in automatic mode to learn the Büchi automaton given by aut.ba is:
<code>
java -jar ROLL.jar aut.ba -aut -table -periodic -under
</code>

==== Data Structures ====

In ROLL,  you can specify the data structures for the learning procedure in the following.

  *  -table. This argument calls for the use of the observation table.
  *  -tree. It requires to use the classification tree for the learner.

==== Learning Algorithms ====

You can also use three canonical FDFAs to learn the Büchi automata as follows.

  * -periodic. Learning the periodic FDFA.
  * -syntactic. Learning the syntactic FDFA.
  * -recurrent. Learning the recurrent FDFA.

If you want to learn the Büchi automaton using L<sub>$</sub> automaton, you should use -ldollar instead of above arguments. We remark that the DFA learning algorithm used for L<sub>$</sub> automaton is an improved version of L<sup>*</sup>.
<code>
java -jar ROLL.jar aut.ba -aut -table -ldollar
</code>

==== From FDFA to BA ====

In ROLL, you can also specify the method to transform an FDFA to a Büchi automaton. There are two methods.


  * -under. This method under approximates the ultimately periodic words accepted by the FDFA.
  * -over. It over approximates the ultimately periodic words accepted by the FDFA.

=== Limit Deterministic Büchi Automaton ===

You can also get a limit deterministic Büchi automaton if you use -ldba along with -under like
<code>
java -jar ROLL.jar aut.ba -aut -table -periodic -under -ldba
</code>

===== Optimizations =====
By default, we use DFA minimization after we build the product of the DFAs in all learning algorithms (including L<sub>$</sub>). 

=== NFA Simulation ===

You can also use the simulation techniques in RABIT to reduce the size of the DFAs as much as possible, which
, however, will produce a NFA. Following is an example:
<code>
java -jar ROLL.jar aut.ba -aut  -table -periodic -sim
</code>


=== Lazy Equivalence Check ===

If you want to reuse the counterexample returned by the teacher as much as possible, you can use the
lazy equivalence check optimization as follows.
<code>
java -jar ROLL.jar aut.ba -aut  -table -periodic -lazyeq
</code>

===== Automata Format =====

ROLL accepts an automaton in the BA format defined on the webpage of RABIT tool. We have also added the support for converting an automaton in the HOAF format to the BA format by relating every propositional formula in HOAF file with a fresh symbol. 

In order to reproduce the experimental results we obtained in the paper, please use the Büchi automata from  [[http://buchi.im.ntu.edu.tw/|Büchi Store]]. 
The command line used in the experiments was in the following form.
<code>
java -jar ROLL.jar aut.hoa -aut -table -periodic -under
</code>
We only changed the data structure and the BA construction method for different algorithms.
We added the argument -lazyeq if we wanted to reuse the counterexample.

We thank the authors of Büchi Store for providing us the examples. The ROLL
library is also inspired a lot from LearnLib (http://learnlib.de/) and libalf (http://libalf.informatik.rwth-aachen.de/),
which are learning libraries for regular language. The Graph package and Value  package are adapted from
IscasMC (http://iscasmc.ios.ac.cn/) by Ernst Moritz Hahn.

Note that ROLL is still at an early stage, we may change the data structures
and refactor the code in the furture.


===== Contacts =====

The ROLL library is maintained by :

Yong Li (liyong@ios.ac.cn)

Depeng Liu
Following people are also involved in the ROLL project:

Yu-Fang Chen
Lijun Zhang
Questions, comments and suggestions are welcome to be sent to one of the developers.

Address for visiting: Building 5, Software Park, No.4, South 4th Street, Zhongguancun, Haidian District, Beijing, China

Zip Code: 100190
